/*
 * tmos - pommax2.c
 * 	version 0.1.0
 * 
 * Copyright (C) 2009..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>
#include <tmos/crc16.h>
#include <tmos/jiffies.h>
#include <tmos/port.h>
#include <stdlib.h>
#include <limits.h>
#include <tmos/adc.h>
#include <tmos/watchdog.h>
#include <tmos/serial.h>
#include <tmos/spi.h>
#include <asm/aduc841.h>

/*
 * POMMAX2 use synchronous serial link.
 *
 * Frame:
 * POMMAX2_SYNC <CH_0> <CH_1> ... <CH_MAX> <CHECKSUM> POMMAX2_STOP
 *
 * POMMAX2_SYNC		1 1 1 1 1 1 1 1
 * <CH_i>		0 0 x x x x x x  0 0 x x x x x x
 * CHECKSUM		0 0 x x x x x x
 * POMMAX_STOP		0 0 0 0 0 0 0 0
 *
 * CHECKSUM is bitwise negation of two's complement sum of CH_i bytes.
 */
#define POMMAX2_SYNC	0xff
#define POMMAX2_STOP	0x00

#define CHANNELS CONFIG_POMMAX2_CHANNELS

/*
 * pommax2_send() is used to send byte. 
 */
#if CONFIG_POMMAX2_SPI
#define pommax2_start(c) spi_start(c)
#define pommax2_send(c) spi_send(c)
#define pommax2_stop() spi_stop()
#elif CONFIG_POMMAX2_SERIAL
#define pommax2_start(c) ser_txc(c)
#define pommax2_send(c) ser_txc(c)
#define pommax2_stop() (0)
#else
#define pommax2_start(c) do { (void) (c); } while (0)
#define pommax2_send(c) do { (void) (c); } while (0)
#define pommax2_stop() (0)
#endif

static unsigned char pommax_send_sample(unsigned char ch)
{
	unsigned char csum = 0;
	int16_t v;
	uint8_t x;

	adc_wait_ch(ch);
	v = adc_get(ch);

	/*
	 * The POMMAX interface allows for 12-bit sample
	 * resolution, but the ADC interface uses 16-bit
	 * format. The four least significant bits are
	 * just skipped.
	 *
	 * TODO: add rounding and error feedback?
	 */
	x = (v >> 10);
	pommax2_send(x & 0x3f);
	csum += (x & 0x3f);

	x = (v >> 4);
	pommax2_send(x & 0x3f);
	csum += (x & 0x3f);
	return csum;
}

void app_main(void) REENTRANT
{
	uint8_t i;
	uint8_t csum;
	uint8_t seq = 0;

	i = 0;
	for (;;) {
		watchdog_refresh();

		seq = adc_sync(seq);
		pommax2_start(POMMAX2_SYNC);
		csum = 0;
		for (i = 0; i < CHANNELS; i++)
			csum += pommax_send_sample(i);
		pommax2_send((~csum) & 0x3f);
		pommax2_send(POMMAX2_STOP);
		pommax2_stop();
	}
}
